const Generator = require('yeoman-generator');
const chalk = require('chalk');
const fetch = require('node-fetch');
const fs = require('fs');
const shell = require('shelljs');
const camelCase = require('camelcase');
const paramCase = require('param-case')

var settings = {
    projName: '',
    camelCaseAppName: '',
    appName: '',
    appDescription: '',
    appVersion: '1.0.0',
    lumenVersion: 'master',
    doGitSetup: false,
    gitRemote: '',
    doDBSetup: false,
    dbName: '',
    dbUser: '',
    dbPassword: ''
};

module.exports = class extends Generator {
    constructor(args, opts) {
        super(args, opts);

        this.argument('projName', { type: String, required: true, description: "The name of the project." });

        settings.projName = this.options.projName;
    }

    prompting() {
        return this.prompt(
            [{
                type: 'input',
                name: 'appName',
                message: 'Application name(eg. vendor/app-name)?',
                default: 'flex/' + paramCase(settings.projName)
            },
            {
                type: 'input',
                name: 'appDescription',
                message: 'What does the app do?',
            },
            {
                type: 'confirm',
                name: 'doGitSetup',
                message: 'Would you like to setup a git repository?'
            },
            {
                when: function (props) {
                    return props.doGitSetup;
                },
                type: 'input',
                name: 'gitRemote',
                message: 'Git remote (leave empty for none)',
                default: ''
            },
            {
                type: 'confirm',
                name: 'doDBSetup',
                message: 'Would you like to setup the Database?'
            },
            {
                when: function (props) {
                    return props.doDBSetup;
                },
                type: 'input',
                name: 'dbName',
                message: 'What is the database name?',
                default: ''
            },
            {
                when: function (props) {
                    return props.doDBSetup;
                },
                type: 'input',
                name: 'dbUser',
                message: 'What is the database user?',
                default: ''
            },
            {
                when: function (props) {
                    return props.doDBSetup;
                },
                type: 'input',
                name: 'dbPassword',
                message: 'What is the database password?',
                default: ''
            }]
        ).then((answers) => {
            settings.appName = answers.appName;
            settings.camelCaseAppName = camelCase(answers.appName.substring(answers.appName.indexOf("/") + 1), { pascalCase: true });

            settings.appDescription = answers.appDescription;
            settings.doGitSetup = answers.doGitSetup;
            settings.gitRemote = answers.gitRemote;
            settings.doDBSetup = answers.doDBSetup;
            settings.dbName = answers.dbName;
            settings.dbUser = answers.dbUser;
            settings.dbPassword = answers.dbPassword;
        });
    }

    lumenDownload() {
        var done = this.async();
        var url = 'https://github.com/laravel/lumen/archive/' + settings.lumenVersion + '.tar.gz';

        this.log.write(chalk.green('Downloading ' + url + '\n'));

        fetch(url)
            .then(res => {
                const dest = fs.createWriteStream('./lumen.tar.gz');
                res.body.pipe(dest);
                //wait for download to actually finish to continue
                setTimeout(done, 1000);
            });
    }

    lumenExtract() {
        this.log(chalk.blue('Extracting lumen.tar.gz\n'));
        let extractCmd = 'tar xvf ./lumen.tar.gz';
        shell.exec(extractCmd);
        shell.mkdir(settings.projName);
        shell.mv('./lumen-master/*', settings.projName);
        shell.rm('-rf', './lumen-master/', 'lumen.tar.gz');
    }

    setupDocker() {
        this.log(chalk.green.bold('Setting up docker folder structure\n'));

        this.fs.copy(
            this.templatePath('_.docker/_nginx/_sites/_default.conf'),
            this.destinationPath(settings.projName + '/.docker/nginx/sites/default.conf')
        );
        this.fs.copy(
            this.templatePath('_.docker/_nginx/_Dockerfile'),
            this.destinationPath(settings.projName + '/.docker/nginx/Dockerfile')
        );
        this.fs.copy(
            this.templatePath('_.docker/_nginx/_http.conf'),
            this.destinationPath(settings.projName + '/.docker/nginx/http.conf')
        );
        this.fs.copy(
            this.templatePath('_.docker/_nginx/_nginx.conf'),
            this.destinationPath(settings.projName + '/.docker/nginx/nginx.conf')
        );
        this.fs.copy(
            this.templatePath('_.docker/_phpfpm/_Dockerfile'),
            this.destinationPath(settings.projName + '/.docker/phpfpm/Dockerfile')
        );
        this.fs.copy(
            this.templatePath('_.docker/_phpfpm/_fpm.conf'),
            this.destinationPath(settings.projName + '/.docker/phpfpm/fpm.conf')
        );
        this.fs.copy(
            this.templatePath('_.docker/_phpfpm/_php.ini'),
            this.destinationPath(settings.projName + '/.docker/phpfpm/php.ini')
        );
        this.fs.copy(
            this.templatePath('_.dockerignore'),
            this.destinationPath(settings.projName + '/.dockerignore')
        );
    }

    setupEnv() {
        this.log(chalk.green.bold('Setting up the rest\n'));

        this.fs.copyTpl(
            this.templatePath('_.env'),
            this.destinationPath(settings.projName + '/.env'),
            {
                appName: settings.camelCaseAppName,
                dbName: settings.dbName,
                dbUser: settings.dbUser,
                dbPassword: settings.dbPassword
            }
        );
        this.fs.copy(
            this.templatePath('_.gitignore'),
            this.destinationPath(settings.projName + '/.gitignore')
        );
        this.fs.copy(
            this.templatePath('_docker-compose.yml'),
            this.destinationPath(settings.projName + '/docker-compose.yml')
        );
    }

    cleanup() {
        shell.cd(settings.projName);
        shell.rm('-rf',
            'database',
            'app/User.php',
            'app/Events',
            'app/Listeners',
            'app/Jobs',
            'app/Middleware',
            'app/Http/Controllers/ExampleController.php',
            'app/Http/Controllers/Controller.php',
            'app/Http/Middleware/ExampleMiddleware.php',
            'app/Providers/*',
            'bootstrap/app.php',
            'composer.json',
            'resources',
            'tests/*',
            'artisan'
        );

        this.fs.copy(
            this.templatePath('_.gitkeep'),
            this.destinationPath(settings.projName + '/app/Providers/.gitkeep')
        );
    }

    setupTests() {
	this.fs.copyTpl(
		this.templatePath('_tests/_StartTest.php'),
		this.destinationPath(settings.projName + '/tests/' + settings.camelCaseAppName + 'Test.php'),
		{
			appName: settings.camelCaseAppName
		}
	);

	this.fs.copy(
		this.templatePath('_tests/_TestCase.php'),
		this.destinationPath(settings.projName + '/tests/TestCase.php')
	);
    }

    setupApp() {
        this.log(chalk.green.bold('Setting up app folder structure\n'));

        shell.mkdir('.phan');

        this.fs.copyTpl(
            this.templatePath('_app/_Http/_Controllers/_StartingController.php'),
            this.destinationPath(settings.projName + '/app/Http/Controllers/' + settings.camelCaseAppName + 'Controller.php'),
            {
                appName: settings.camelCaseAppName,
            }
        );

        this.fs.copyTpl(
            this.templatePath('_composer.json'),
            this.destinationPath(settings.projName + '/composer.json'),
            {
                appName: settings.appName,
                appDescription: settings.appDescription
            }
        );

        this.fs.copy(
            this.templatePath('_app/_Http/_Controllers/_BaseController.php'),
            this.destinationPath(settings.projName + '/app/Http/Controllers/BaseController.php')
        );
        this.fs.copy(
            this.templatePath('_app/_Exception/_Handler.php'),
            this.destinationPath(settings.projName + '/app/Exception/Handler.php')
        );
        this.fs.copy(
            this.templatePath('_bootstrap/_app.php'),
            this.destinationPath(settings.projName + '/bootstrap/app.php')
        );
        this.fs.copy(
            this.templatePath('_.phan/_config.php'),
            this.destinationPath(settings.projName + '/.phan/config.php')
        );
        this.fs.copy(
            this.templatePath('_phpcs.xml'),
            this.destinationPath(settings.projName + '/phpcs.xml')
        );
    }

    setupGit() {
        var done = this.async();

        if (settings.doGitSetup) {
            this.log(chalk.green.bold('Setting up empty git repo\n'));

            shell.cd(settings.projName);

            var git = shell.exec('git init', { silent: true });

            if (git.code !== 0) {
                this.log(chalk.red.bold(git.output + '\n'));
            } else {
                shell.exec('git add --all', { silent: true });
                shell.exec('git commit -m \'Initial Commit\'', { silent: true });

                if (settings.gitRemote !== '') {
                    var remote = shell.exec('git remote add origin ' + settings.gitRemote);

                    if (remote.code !== 0) {
                        this.log(chalk.red.bold(remote.output + '\n'));
                    } else {
                        this.log(chalk.green.bold('git remote added\n'));
                    }
                }
            }
        }

        done();
    }

    installComposer() {
        var done = this.async();
        this.log(chalk.green.bold('Installing composer dependancies\n'));

        shell.cd(settings.projName);
        shell.exec('composer install');

        done();
    }
};
